@@ -10,18 +10,18 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import ddt
+from http import HTTPStatus
 import inspect
+from unittest import mock
 
-import mock
+import ddt
 from oslo_utils import encodeutils
-from six.moves import http_client
 import webob
 
 from cinder.api.openstack import wsgi
 from cinder import exception
-from cinder import test
 from cinder.tests.unit.api import fakes
+from cinder.tests.unit import test
 
 
 class RequestTest(test.TestCase):
@@ -234,7 +234,7 @@ class ResourceTest(test.TestCase):
         app = fakes.TestRouter(Controller())
         response = req.get_response(app)
         self.assertEqual(b'off', response.body)
-        self.assertEqual(http_client.OK, response.status_int)
+        self.assertEqual(HTTPStatus.OK, response.status_int)
 
     def test_resource_not_authorized(self):
         class Controller(object):
@@ -244,7 +244,7 @@ class ResourceTest(test.TestCase):
         req = webob.Request.blank('/tests')
         app = fakes.TestRouter(Controller())
         response = req.get_response(app)
-        self.assertEqual(http_client.FORBIDDEN, response.status_int)
+        self.assertEqual(HTTPStatus.FORBIDDEN, response.status_int)
 
     def test_dispatch(self):
         class Controller(object):
@@ -777,21 +777,21 @@ class ResourceTest(test.TestCase):
 class ResponseObjectTest(test.TestCase):
     def test_default_code(self):
         robj = wsgi.ResponseObject({})
-        self.assertEqual(http_client.OK, robj.code)
+        self.assertEqual(HTTPStatus.OK, robj.code)
 
     def test_modified_code(self):
         robj = wsgi.ResponseObject({})
-        robj._default_code = http_client.ACCEPTED
-        self.assertEqual(http_client.ACCEPTED, robj.code)
+        robj._default_code = HTTPStatus.ACCEPTED
+        self.assertEqual(HTTPStatus.ACCEPTED, robj.code)
 
     def test_override_default_code(self):
-        robj = wsgi.ResponseObject({}, code=http_client.NOT_FOUND)
-        self.assertEqual(http_client.NOT_FOUND, robj.code)
+        robj = wsgi.ResponseObject({}, code=HTTPStatus.NOT_FOUND)
+        self.assertEqual(HTTPStatus.NOT_FOUND, robj.code)
 
     def test_override_modified_code(self):
-        robj = wsgi.ResponseObject({}, code=http_client.NOT_FOUND)
-        robj._default_code = http_client.ACCEPTED
-        self.assertEqual(http_client.NOT_FOUND, robj.code)
+        robj = wsgi.ResponseObject({}, code=HTTPStatus.NOT_FOUND)
+        robj._default_code = HTTPStatus.ACCEPTED
+        self.assertEqual(HTTPStatus.NOT_FOUND, robj.code)
 
     def test_set_header(self):
         robj = wsgi.ResponseObject({})
