@@ -15,17 +15,15 @@
 
 
 import datetime
+from http import HTTPStatus
 import json
 from unittest import mock
+import urllib
 
 import ddt
 import fixtures
 import iso8601
 from oslo_config import cfg
-import six
-from six.moves import http_client
-from six.moves import range
-from six.moves import urllib
 import webob
 
 from cinder.api import common
@@ -38,12 +36,12 @@ from cinder import exception
 from cinder import group as groupAPI
 from cinder import objects
 from cinder.objects import fields
-from cinder import test
 from cinder.tests.unit.api import fakes
 from cinder.tests.unit.api.v2 import fakes as v2_fakes
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import fake_volume
 from cinder.tests.unit.image import fake as fake_image
+from cinder.tests.unit import test
 from cinder.tests.unit import utils
 from cinder.volume import api as volume_api
 
@@ -160,8 +158,7 @@ class VolumeApiTest(test.TestCase):
                              consistencygroup_id=None,
                              volume_type=None,
                              image_ref=None,
-                             image_id=None,
-                             multiattach=False):
+                             image_id=None):
         vol = {"size": size,
                "name": name,
                "description": description,
@@ -170,7 +167,6 @@ class VolumeApiTest(test.TestCase):
                "source_volid": source_volid,
                "consistencygroup_id": consistencygroup_id,
                "volume_type": volume_type,
-               "multiattach": multiattach,
                }
 
         if image_id is not None:
@@ -242,7 +238,6 @@ class VolumeApiTest(test.TestCase):
                 'consistencygroup': None,
                 'availability_zone': availability_zone,
                 'scheduler_hints': None,
-                'multiattach': False,
                 }
 
     @mock.patch.object(db.sqlalchemy.api, '_volume_type_get_full',
@@ -572,37 +567,6 @@ class VolumeApiTest(test.TestCase):
                           req,
                           body=body)
 
-    def test_volume_create_with_invalid_multiattach(self):
-        vol = self._vol_in_request_body(multiattach="InvalidBool")
-        body = {"volume": vol}
-        req = fakes.HTTPRequest.blank('/v2/volumes')
-
-        self.assertRaises(exception.ValidationError,
-                          self.controller.create,
-                          req,
-                          body=body)
-
-    @mock.patch.object(volume_api.API, 'create', autospec=True)
-    @mock.patch.object(volume_api.API, 'get', autospec=True)
-    @mock.patch.object(db.sqlalchemy.api, '_volume_type_get_full',
-                       autospec=True)
-    def test_volume_create_with_valid_multiattach(self,
-                                                  volume_type_get,
-                                                  get, create):
-        create.side_effect = v2_fakes.fake_volume_api_create
-        get.side_effect = v2_fakes.fake_volume_get
-        volume_type_get.side_effect = v2_fakes.fake_volume_type_get
-
-        vol = self._vol_in_request_body(multiattach=True)
-        body = {"volume": vol}
-
-        ex = self._expected_vol_from_controller(multiattach=True)
-
-        req = fakes.HTTPRequest.blank('/v2/volumes')
-        res_dict = self.controller.create(req, body=body)
-
-        self.assertEqual(ex, res_dict)
-
     @ddt.data({'a' * 256: 'a'},
               {'a': 'a' * 256},
               {'': 'a'},
@@ -917,6 +881,33 @@ class VolumeApiTest(test.TestCase):
         expected = {'volumes': [exp_vol['volume']]}
         self.assertEqual(expected, res_dict)
 
+    def test_volume_list_detail_host_name_admin_non_admin(self):
+        fake_host = 'fake_host'
+        volume = v2_fakes.create_fake_volume(fake.VOLUME_ID)
+        del volume['name']
+        del volume['volume_type']
+        db.volume_create(context.get_admin_context(), volume)
+        values = {'volume_id': fake.VOLUME_ID, }
+        attachment = db.volume_attach(context.get_admin_context(), values)
+        db.volume_attached(context.get_admin_context(),
+                           attachment['id'], fake.INSTANCE_ID, fake_host, '/')
+        db.volume_attachment_get(context.get_admin_context(),
+                                 attachment['id'])
+
+        req = fakes.HTTPRequest.blank('/v2/volumes/detail')
+        res_dict = self.controller.detail(req)
+        # host_name will always be None for non-admins
+        self.assertIsNone(
+            res_dict['volumes'][0]['attachments'][0]['host_name'])
+
+        admin_ctx = context.RequestContext(fake.USER_ID, fake.PROJECT_ID, True)
+        req.environ['cinder.context'] = admin_ctx
+        res_dict = self.controller.detail(req)
+        # correct host_name is returned for admins
+        self.assertEqual(fake_host,
+                         res_dict['volumes'][0]['attachments'][0]['host_name']
+                         )
+
     def test_volume_index_with_marker(self):
         def fake_volume_get_all_by_project(context, project_id, marker, limit,
                                            sort_keys=None, sort_dirs=None,
@@ -1419,7 +1410,7 @@ class VolumeApiTest(test.TestCase):
     def test_volume_delete(self):
         req = fakes.HTTPRequest.blank('/v2/volumes/%s' % fake.VOLUME_ID)
         resp = self.controller.delete(req, fake.VOLUME_ID)
-        self.assertEqual(http_client.ACCEPTED, resp.status_int)
+        self.assertEqual(HTTPStatus.ACCEPTED, resp.status_int)
 
     def test_volume_delete_attached(self):
         def fake_volume_attached(self, context, volume,
@@ -1433,7 +1424,7 @@ class VolumeApiTest(test.TestCase):
                                 self.controller.delete,
                                 req, 1)
         expect_msg = "Volume 1 is still attached, detach volume first."
-        self.assertEqual(expect_msg, six.text_type(exp))
+        self.assertEqual(expect_msg, str(exp))
 
     def test_volume_delete_no_volume(self):
         self.mock_object(volume_api.API, "get",
