@@ -22,13 +22,14 @@ import requests
 from requests import models
 
 from cinder import context as cinder_context
-from cinder import db
 from cinder.db.sqlalchemy import api as sqlalchemy_api
+from cinder.objects import group_snapshot as obj_group_snap
 from cinder.objects import snapshot as obj_snap
+from cinder.tests.unit import fake_group
+from cinder.tests.unit import fake_group_snapshot
 from cinder.tests.unit import fake_snapshot
 from cinder.tests.unit import fake_volume
 from cinder.tests.unit import test
-from cinder import utils
 from cinder.volume import configuration as conf
 from cinder.volume import driver
 from cinder.volume.drivers.hitachi import hbsd_common
@@ -37,6 +38,7 @@ from cinder.volume.drivers.hitachi import hbsd_rest
 from cinder.volume.drivers.hitachi import hbsd_rest_api
 from cinder.volume.drivers.hitachi import hbsd_utils
 from cinder.volume import volume_types
+from cinder.volume import volume_utils
 from cinder.zonemanager import utils as fczm_utils
 
 # Configuration parameter values
@@ -73,11 +75,14 @@ DEFAULT_CONNECTOR = {
 CTXT = cinder_context.get_admin_context()
 
 TEST_VOLUME = []
-for i in range(3):
+for i in range(4):
     volume = {}
     volume['id'] = '00000000-0000-0000-0000-{0:012d}'.format(i)
     volume['name'] = 'test-volume{0:d}'.format(i)
-    volume['provider_location'] = '{0:d}'.format(i)
+    if i == 3:
+        volume['provider_location'] = None
+    else:
+        volume['provider_location'] = '{0:d}'.format(i)
     volume['size'] = 128
     if i == 2:
         volume['status'] = 'in-use'
@@ -107,6 +112,23 @@ snapshot = obj_snap.Snapshot._from_db_object(
     fake_snapshot.fake_db_snapshot(**snapshot))
 TEST_SNAPSHOT.append(snapshot)
 
+TEST_GROUP = []
+for i in range(2):
+    group = {}
+    group['id'] = '20000000-0000-0000-0000-{0:012d}'.format(i)
+    group['status'] = 'available'
+    group = fake_group.fake_group_obj(CTXT, **group)
+    TEST_GROUP.append(group)
+
+TEST_GROUP_SNAP = []
+group_snapshot = {}
+group_snapshot['id'] = '30000000-0000-0000-0000-{0:012d}'.format(0)
+group_snapshot['status'] = 'available'
+group_snapshot = obj_group_snap.GroupSnapshot._from_db_object(
+    CTXT, obj_group_snap.GroupSnapshot(),
+    fake_group_snapshot.fake_db_group_snapshot(**group_snapshot))
+TEST_GROUP_SNAP.append(group_snapshot)
+
 # Dummy response for REST API
 POST_SESSIONS_RESULT = {
     "token": "b74777a3-f9f0-4ea8-bd8f-09847fac48d3",
@@ -205,6 +227,18 @@ GET_SNAPSHOTS_RESULT = {
     ],
 }
 
+GET_SNAPSHOTS_RESULT_PAIR = {
+    "data": [
+        {
+            "primaryOrSecondary": "S-VOL",
+            "status": "PAIR",
+            "pvolLdevId": 0,
+            "muNumber": 1,
+            "svolLdevId": 1,
+        },
+    ],
+}
+
 GET_SNAPSHOTS_RESULT_BUSY = {
     "data": [
         {
@@ -403,6 +437,7 @@ class HBSDRESTFCDriverTest(test.TestCase):
             'rest_server_ip_port']
         self.configuration.hitachi_rest_tcp_keepalive = True
         self.configuration.hitachi_discard_zero_page = True
+        self.configuration.hitachi_rest_number = "0"
 
         self.configuration.hitachi_zoning_request = False
 
@@ -434,13 +469,13 @@ class HBSDRESTFCDriverTest(test.TestCase):
 
     @mock.patch.object(requests.Session, "request")
     @mock.patch.object(
-        utils, 'brick_get_connector_properties',
+        volume_utils, 'brick_get_connector_properties',
         side_effect=_brick_get_connector_properties)
     def _setup_driver(
             self, brick_get_connector_properties=None, request=None):
         """Set up the driver environment."""
         self.driver = hbsd_fc.HBSDFCDriver(
-            configuration=self.configuration, db=db)
+            configuration=self.configuration)
         request.side_effect = [FakeResponse(200, POST_SESSIONS_RESULT),
                                FakeResponse(200, GET_PORTS_RESULT),
                                FakeResponse(200, GET_HOST_WWNS_RESULT)]
@@ -462,11 +497,11 @@ class HBSDRESTFCDriverTest(test.TestCase):
     # API test cases
     @mock.patch.object(requests.Session, "request")
     @mock.patch.object(
-        utils, 'brick_get_connector_properties',
+        volume_utils, 'brick_get_connector_properties',
         side_effect=_brick_get_connector_properties)
     def test_do_setup(self, brick_get_connector_properties, request):
         drv = hbsd_fc.HBSDFCDriver(
-            configuration=self.configuration, db=db)
+            configuration=self.configuration)
         self._setup_config()
         request.side_effect = [FakeResponse(200, POST_SESSIONS_RESULT),
                                FakeResponse(200, GET_PORTS_RESULT),
@@ -483,12 +518,12 @@ class HBSDRESTFCDriverTest(test.TestCase):
 
     @mock.patch.object(requests.Session, "request")
     @mock.patch.object(
-        utils, 'brick_get_connector_properties',
+        volume_utils, 'brick_get_connector_properties',
         side_effect=_brick_get_connector_properties)
     def test_do_setup_create_hg(self, brick_get_connector_properties, request):
         """Normal case: The host group not exists."""
         drv = hbsd_fc.HBSDFCDriver(
-            configuration=self.configuration, db=db)
+            configuration=self.configuration)
         self._setup_config()
         request.side_effect = [FakeResponse(200, POST_SESSIONS_RESULT),
                                FakeResponse(200, GET_PORTS_RESULT),
@@ -510,12 +545,12 @@ class HBSDRESTFCDriverTest(test.TestCase):
 
     @mock.patch.object(requests.Session, "request")
     @mock.patch.object(
-        utils, 'brick_get_connector_properties',
+        volume_utils, 'brick_get_connector_properties',
         side_effect=_brick_get_connector_properties)
     def test_do_setup_pool_name(self, brick_get_connector_properties, request):
         """Normal case: Specify a pool name instead of pool id"""
         drv = hbsd_fc.HBSDFCDriver(
-            configuration=self.configuration, db=db)
+            configuration=self.configuration)
         self._setup_config()
         tmp_pool = self.configuration.hitachi_pool
         self.configuration.hitachi_pool = CONFIG_MAP['pool_name']
@@ -896,3 +931,143 @@ class HBSDRESTFCDriverTest(test.TestCase):
         req = models.Response()
         ret = session.__call__(req)
         self.assertEqual('Session token', ret.headers['Authorization'])
+
+    def test_create_group(self):
+        ret = self.driver.create_group(self.ctxt, TEST_GROUP[0])
+        self.assertIsNone(ret)
+
+    @mock.patch.object(requests.Session, "request")
+    def test_delete_group(self, request):
+        request.side_effect = [FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT)]
+        ret = self.driver.delete_group(
+            self.ctxt, TEST_GROUP[0], [TEST_VOLUME[0]])
+        self.assertEqual(4, request.call_count)
+        actual = (
+            {'status': TEST_GROUP[0]['status']},
+            [{'id': TEST_VOLUME[0]['id'], 'status': 'deleted'}]
+        )
+        self.assertTupleEqual(actual, ret)
+
+    @mock.patch.object(requests.Session, "request")
+    def test_create_group_from_src_volume(self, request):
+        request.side_effect = [FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(200, GET_SNAPSHOTS_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT)]
+        ret = self.driver.create_group_from_src(
+            self.ctxt, TEST_GROUP[1], [TEST_VOLUME[1]],
+            source_group=TEST_GROUP[0], source_vols=[TEST_VOLUME[0]]
+        )
+        self.assertEqual(5, request.call_count)
+        actual = (
+            None, [{'id': TEST_VOLUME[1]['id'], 'provider_location': '1'}])
+        self.assertTupleEqual(actual, ret)
+
+    @mock.patch.object(requests.Session, "request")
+    def test_create_group_from_src_snapshot(self, request):
+        request.side_effect = [FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(200, GET_SNAPSHOTS_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT)]
+        ret = self.driver.create_group_from_src(
+            self.ctxt, TEST_GROUP[0], [TEST_VOLUME[0]],
+            group_snapshot=TEST_GROUP_SNAP[0], snapshots=[TEST_SNAPSHOT[0]]
+        )
+        self.assertEqual(5, request.call_count)
+        actual = (
+            None, [{'id': TEST_VOLUME[0]['id'], 'provider_location': '1'}])
+        self.assertTupleEqual(actual, ret)
+
+    def test_create_group_from_src_volume_error(self):
+        self.assertRaises(
+            hbsd_utils.HBSDError, self.driver.create_group_from_src,
+            self.ctxt, TEST_GROUP[1], [TEST_VOLUME[1]],
+            source_group=TEST_GROUP[0], source_vols=[TEST_VOLUME[3]]
+        )
+
+    @mock.patch.object(volume_utils, 'is_group_a_cg_snapshot_type')
+    def test_update_group(self, is_group_a_cg_snapshot_type):
+        is_group_a_cg_snapshot_type.return_value = False
+        ret = self.driver.update_group(
+            self.ctxt, TEST_GROUP[0], add_volumes=[TEST_VOLUME[0]])
+        self.assertTupleEqual((None, None, None), ret)
+
+    @mock.patch.object(volume_utils, 'is_group_a_cg_snapshot_type')
+    def test_update_group_error(self, is_group_a_cg_snapshot_type):
+        is_group_a_cg_snapshot_type.return_value = True
+        self.assertRaises(
+            hbsd_utils.HBSDError, self.driver.update_group,
+            self.ctxt, TEST_GROUP[0], add_volumes=[TEST_VOLUME[3]],
+            remove_volumes=[TEST_VOLUME[0]]
+        )
+
+    @mock.patch.object(requests.Session, "request")
+    @mock.patch.object(sqlalchemy_api, 'volume_get', side_effect=_volume_get)
+    @mock.patch.object(volume_utils, 'is_group_a_cg_snapshot_type')
+    def test_create_group_snapshot_non_cg(
+            self, is_group_a_cg_snapshot_type, volume_get, request):
+        is_group_a_cg_snapshot_type.return_value = False
+        request.side_effect = [FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(200, GET_SNAPSHOTS_RESULT)]
+        ret = self.driver.create_group_snapshot(
+            self.ctxt, TEST_GROUP_SNAP[0], [TEST_SNAPSHOT[0]]
+        )
+        self.assertEqual(4, request.call_count)
+        actual = (
+            {'status': 'available'},
+            [{'id': TEST_SNAPSHOT[0]['id'],
+              'provider_location': '1',
+              'status': 'available'}]
+        )
+        self.assertTupleEqual(actual, ret)
+
+    @mock.patch.object(requests.Session, "request")
+    @mock.patch.object(sqlalchemy_api, 'volume_get', side_effect=_volume_get)
+    @mock.patch.object(volume_utils, 'is_group_a_cg_snapshot_type')
+    def test_create_group_snapshot_cg(
+            self, is_group_a_cg_snapshot_type, volume_get, request):
+        is_group_a_cg_snapshot_type.return_value = True
+        request.side_effect = [FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(200, GET_SNAPSHOTS_RESULT_PAIR),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(200, GET_SNAPSHOTS_RESULT)]
+        ret = self.driver.create_group_snapshot(
+            self.ctxt, TEST_GROUP_SNAP[0], [TEST_SNAPSHOT[0]]
+        )
+        self.assertEqual(5, request.call_count)
+        actual = (
+            None,
+            [{'id': TEST_SNAPSHOT[0]['id'],
+              'provider_location': '1',
+              'status': 'available'}]
+        )
+        self.assertTupleEqual(actual, ret)
+
+    @mock.patch.object(requests.Session, "request")
+    def test_delete_group_snapshot(self, request):
+        request.side_effect = [FakeResponse(200, GET_LDEV_RESULT_PAIR),
+                               FakeResponse(200, NOTFOUND_RESULT),
+                               FakeResponse(200, GET_SNAPSHOTS_RESULT),
+                               FakeResponse(200, GET_SNAPSHOTS_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT)]
+        ret = self.driver.delete_group_snapshot(
+            self.ctxt, TEST_GROUP_SNAP[0], [TEST_SNAPSHOT[0]])
+        self.assertEqual(10, request.call_count)
+        actual = (
+            {'status': TEST_GROUP_SNAP[0]['status']},
+            [{'id': TEST_SNAPSHOT[0]['id'], 'status': 'deleted'}]
+        )
+        self.assertTupleEqual(actual, ret)
